unit QfelPopUpHelp;

{}
{ (C)1997 by Marcin Zaleski                     }
{ Partially based on PopHelp.pas from Triplex++ }
{}

interface

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls;

type

  TUseTags = (utNever,utGraphicControl,utAlways);

  TQfelPopUpHelp = class(TComponent)
    private
      FOldFormClick  : TNotifyEvent;
      FOldForm       : TKeyEvent;
      FOldMouse      : TCursor;
      FOldPreview    : Boolean;
      FCurControl    : TControl;
      FActiveForm    : TForm;
      FActiveHandle  : THandle;
      FControlPos    : TPoint;
      FHelpId        : Integer;
      FViaMouse      : Boolean;
      FOldDeact      : TNotifyEvent;
      FOldDblClick   : TNotifyEvent;
      FUseTags       : TUseTags;
      FCancelOnZero : Boolean;
      procedure AppMessage(var Msg:TMsg; var Handled:Boolean);
      procedure FormKeyDown(Sender:TObject;var Key: Word;Shift: TShiftState);
      procedure FormDblClick(Sender:TObject);
      procedure FormClick(Sender:TObject);
      procedure FormDeactivate(Sender:TObject);
      procedure CheckHelp;
    public
      constructor Create(AOwner:TComponent); virtual;
      procedure DisplayHelp;
      procedure UseMouse;
      procedure CancelHelp;
      procedure UseFocus;
    published
      property CancelOnZero:Boolean read FCancelOnZero write FCancelOnZero;
      property UseTags:TUseTags read FUseTags write FUseTags;
  end;

procedure Register;

implementation

constructor TQfelPopUpHelp.Create(AOwner:TComponent);
begin
  inherited Create(AOwner);
  CancelOnZero := True;
  UseTags := utGraphicControl;
end;

procedure TQfelPopUpHelp.UseMouse;
begin
  FOldMouse := Screen.Cursor;
  FViaMouse := True;
  FActiveForm := Screen.ActiveForm;
  FActiveHandle := FActiveForm.Handle;
  with FActiveForm do
    begin
      SetCaptureControl(FActiveForm);
      ControlStyle := ControlStyle - [csCaptureMouse];
      FoldFormClick := OnClick;
      OnClick := FormClick;
      FOldDblClick := OnDblClick;
      OnDblClick := FormDblClick;
      FOldForm := OnKeyDown;
      OnKeyDown := FormKeyDown;
      FOldPreview := KeyPreview;
      KeyPreview := True;
    end;
  FOldDeact := Application.OnDeactivate;
  Application.OnDeactivate := FormDeactivate;
  Screen.Cursor := crHelp;
end;

procedure TQfelPopUpHelp.CancelHelp;
begin
  with FActiveForm do
    begin
      SetCaptureControl(nil);
      ControlStyle := ControlStyle + [csCaptureMouse];
      OnClick := FOldFormClick;
      OnDblClick := FOldDblClick;
      OnKeyDown := FOldForm;
      KeyPreview := FOldPreview;
    end;
  Application.OnDeactivate := FOldDeact;
  Screen.Cursor := FOldMouse;
end;

procedure TQfelPopUpHelp.UseFocus;
var
  Rect : TRect;
begin
  FCurControl := Screen.ActiveControl;
  GetWindowRect(FCurControl.Parent.Handle,Rect);
  with FCurControl do
    begin
      FControlPos.X := Rect.Left + (Left + (Width div 2));
      FControlPos.Y := Rect.Top + (Top + Height);
    end;
  CheckHelp;
end;

procedure TQfelPopUpHelp.CheckHelp;
begin
  case FUseTags of
    utNever:
      begin
        if FCurControl is TWinControl then
          FHelpId := (FCurControl as TWinControl).HelpContext
        else
          FHelpId := 0;
      end;
    utGraphicControl:
      begin
        if FCurControl is TGraphicControl then
          FHelpId := FCurControl.Tag
        else if FCurControl is TWinControl then
          FHelpId := (FCurControl as TWinControl).HelpContext
        else
          FHelpId := 0;
      end;
    utAlways:
      begin
        if FCurControl is TGraphicControl then
          FHelpId := FCurControl.Tag
        else if FCurControl is TWinControl then
          begin
            FHelpId := (FCurControl as TWinControl).HelpContext;
            if FHelpId=0 then
              FHelpId := FCurControl.Tag;
          end
        else
          FHelpId := 0;
      end;
  end;
{
  if FUseTags<>utNever then
    begin
      if FCurControl is TGraphicControl then
        FHelpId := FCurControl.Tag
      else if FCurControl is TWinControl then
        FHelpId := (FCurControl as TWinControl).HelpContext
      else
        FHelpId := 0;
    end
  else
    begin
      if FCurControl is TWinControl then
        FHelpId := (FCurControl as TWinControl).HelpContext
      else
        FHelpId := 0;
    end;}

  if FhelpId<>0 then
    begin
      if FViaMouse then
        CancelHelp;
      DisplayHelp;
    end
  else
    if FCancelOnZero then
      CancelHelp;
end;

procedure TQfelPopUpHelp.FormDeactivate(Sender:TObject);
begin
  CancelHelp;
end;

procedure TQfelPopUpHelp.FormClick(Sender:TObject);
var
  Point   : TPoint;
  Control : TControl;
begin
  GetCursorPos(Point);
  Control := FindDragTarget(Point,True);
  if Control<>nil then
    begin
      FCurControl := Control;
      FControlPos := Point;
      CheckHelp;
    end;
end;

procedure TQfelPopUpHelp.FormDblClick(Sender:TObject);
begin
  FormClick(Sender);
end;

procedure TQfelPopUpHelp.AppMessage(var Msg:TMsg;var Handled:Boolean);
begin
  case Msg.message of
    WM_LBUTTONUP : Handled := True;
    WM_LBUTTONDOWN : Handled := True;
  end;
end;

procedure TQfelPopUpHelp.FormKeyDown(Sender:TObject;var Key:Word;Shift:TShiftState);
begin
  if Key=27 then
    CancelHelp;
end;

procedure TQfelPopUpHelp.DisplayHelp;
begin
  Application.HelpCommand(HELP_CONTEXTPOPUP,FHelpId);
end;

procedure Register;
begin
  RegisterComponents('Qfel',[TQfelPopUpHelp]);
end;

end.
